/* $Id: encryptdl.c,v 1.4.8.2 2017/01/09 16:11:25 tm Exp $
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 *
 * Encryption sample, demonstrating how to load the PLOP DLL dynamically
 */

#include <stdio.h>

#include "ploplibdl.h"

int
main(int argc, char *argv[])
{
    PLOP *plop;
    const PLOP_api *plop_api;

    /* parameters for the input document */
    const char *in_filename = "PLOP-datasheet.pdf";
    const char *in_password = "";

    /* parameters for the output document */
    const char *out_filename = "PLOP-datasheet-encrypted-dl.pdf";
    const char *out_master = "DEMO";
    const char *out_user = "demo";
    const char *permissions = "";

    /* This is where input files live. Adjust as necessary. */
    const char *searchpath = "../data";

    (void) argc;

    /* create a new PLOP object */
    if ((plop_api = PLOP_new_dl(&plop)) == (PLOP_api *) 0)
    {
        fprintf(stderr, "%s: unable to load PLOP DLL\n", argv[0]);
	return(2);
    }

    PLOP_TRY_DL(plop_api, plop) {
	int doc;
	char optlist[1024];

	sprintf(optlist, "searchpath={%s}", searchpath);
	plop_api->PLOP_set_option(plop, optlist);

	/* open unprotected input file without any password */
	sprintf(optlist, "password {%s} ", in_password);
	if ((doc = plop_api->PLOP_open_document
                                    (plop, in_filename, 0, optlist)) == -1)
	{
	    printf("Error: %s\n", plop_api->PLOP_get_errmsg(plop));
	    PLOP_delete_dl(plop_api, plop);
	    return(2);
	}

	/* create the output file with a password */
	sprintf(optlist, "masterpassword={%s} userpassword={%s} "
	    "permissions={%s} input=%d",
	    out_master, out_user, permissions, doc);

	if (plop_api->PLOP_create_document(plop, out_filename, 0, optlist)== -1)
	{
	    printf("Error: %s\n", plop_api->PLOP_get_errmsg(plop));
	    PLOP_delete_dl(plop_api, plop);
	    return(2);
	}

	/* close input and output files */
        plop_api->PLOP_close_document(plop, doc, "");
    }

    PLOP_CATCH_DL(plop_api, plop) {
	printf("PLOP exception occurred in encryptdl:\n");
	printf("[%d] %s: %s\n", plop_api->PLOP_get_errnum(plop),
            plop_api->PLOP_get_apiname(plop), plop_api->PLOP_get_errmsg(plop));
	PLOP_delete_dl(plop_api, plop);
	return(2);
    }

    /* delete the PLOP object */
    PLOP_delete_dl(plop_api, plop);

    return(0);
}
