/* $Id: sign.c,v 1.1.2.2 2017/02/22 14:31:58 rjs Exp $
 *
 * PDFlib PLOP DS:
 * Demonstrate digital signature features with optional signature visualization
 *
 */

#include <stdio.h>
#include <string.h>

#include "ploplib.h"

int
main(int argc, char **argv)
{
    PLOP *plop;

    /* Basic signature options */
    static const char sign_opts[] =
        "ltv=try "          /* use ltv=full to ensure LTV status */
#if defined(USE_PKCS11)
        "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
#else
        "engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} "
        "password={demo} "
#endif
    ;

    /* This is where input files live. Adjust as necessary. */
    const char *searchpath = "../data";

    /* Set to 1 to enable signature visualization */
    int visualize = 0;
    const char *visfile = "signing_man_pdfa1a.pdf";

    if (argc < 3)
    {
	printf("usage: sign <infile> <outfile>\n");
	return 2;
    }

    if ((plop = PLOP_new()) == (PLOP *) 0)
    {
	printf("Couldn't create PLOP object (out of memory)!\n");
	return(2);
    }

    PLOP_TRY(plop)
    {
	int doc, visdoc;
	char optlist[1024];

	sprintf(optlist, "searchpath={%s}", searchpath);
	PLOP_set_option(plop, optlist);

	if ((doc = PLOP_open_document(plop, argv[1], 0, "")) == -1)
	{
	    printf("Error: %s: %s\n", PLOP_get_apiname(plop),
	                                PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    return(2);
	}

	/* Basic signature options */
	strcpy(optlist, sign_opts);

	if (visualize)
	{
	    char fieldoptlist[1024];

            if ((visdoc = PLOP_open_document(plop, visfile, 0, "")) == -1)
            {
                printf("Error: %s\n", PLOP_get_errmsg(plop));
                PLOP_delete(plop);
                return(2);
            }

            /* Add field option for visualization */
            sprintf(fieldoptlist,
                    "field={visdoc=%d rect={50 50 adapt adapt}} ", visdoc);
            strcat(optlist, fieldoptlist);
	}

	if (PLOP_prepare_signature(plop, optlist) == -1)
	{
	    printf("Error: %s: %s\n",
		    PLOP_get_apiname(plop), PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    return(2);
	}

	sprintf(optlist, "input=%d", doc);
	if (PLOP_create_document(plop, argv[2], 0, optlist) == -1)
	{
	    printf("Error: %s: %s\n",
		    PLOP_get_apiname(plop), PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    return(2);
	}

        PLOP_close_document(plop, doc, "");

        if (visualize)
        {
            PLOP_close_document(plop, visdoc, "");
        }
    }

    PLOP_CATCH(plop)
    {
	printf("PLOP exception occurred in sign:\n");
	printf("[%d] %s: %s\n", PLOP_get_errnum(plop),
		PLOP_get_apiname(plop), PLOP_get_errmsg(plop));
	PLOP_delete(plop);
	return(2);
    }

    PLOP_delete(plop);

    return(0);
}
