#!/usr/bin/perl
# $Id: certsec.pl,v 1.5 2016/05/13 19:34:55 rjs Exp $
#
# PDFlib PLOP DS:
# Encrypt file with certificate security.
#

use PDFlib::PLOP 5.2;

# create a new PLOP object
my $plop = new PDFlib::PLOP;
#
# This is where input files live. Adjust as necessary.
my $searchpath = "../data";

my @recipients = (
    # no permission restrictions
    { cert=>"../data/demo_recipient_1.pem", perm=>"" },

    # form filling and signing permissions granted, but not others
    { cert=>"../data/demo_recipient_2.pem", perm=>"nomodify nocopy noannots noassemble" },
    { cert=>"../data/demo_recipient_3.pem", perm=>"nomodify nocopy noannots noassemble" },
);


if ($#ARGV != 1 ) {
    die("usage: certsec <infile> <outfile> \n");
}

eval {
    my $doc;
    my $r;
    my $optlist;

    $plop->set_option("searchpath={${searchpath}}");

    # open unprotected input file
    if (($doc = $plop->open_document($ARGV[0], "")) == -1) {
	die("Error: ", $plop->get_errmsg(), "\n");
    }

    # add recipients
    for ($r = 0; $r <= $#recipients; $r++) {
        $optlist = sprintf("certificate={filename={%s}} permissions={%s}",
            $recipients[$r]{cert}, $recipients[$r]{perm});
        if ($plop->add_recipient($optlist) == -1) {
            # emit warning and continue
            print("Error: ", $plop->get_errmsg(), "\n");
        }
    }

    # create protected output file
    $optlist = sprintf("input=%d", $doc);
    if ($plop->create_document($ARGV[1], $optlist) == -1)
    {
	die("Error: ", $plop->get_errmsg(), "\n");
    }

    # close input and output files
    $plop->close_document($doc, "");
};

if ($@) {
    die("$0: PLOP Exception occurred:\n$@");
}
